load detpi.mat;

sigma   = 1.50;                     % utility parameter
delta   = 0.10;                     % depreciation rate
beta    = 0.95;                     % discount factor
alpha   = 0.30;                     % capital elasticity of output

nbk     = 1000;                     % number of data points in the grid
crit    = 1;                        % convergence criterion
iter    = 1;                        % iteration
epsi    = 1e-6;                     % convergence parameter

ks      = ((1-beta*(1-delta))/(alpha*beta))^(1/(alpha-1));
ys			= ks^alpha;
is			= delta*ks;
cs			= ys-is;

dev     = 0.9;                      % maximal deviation from steady state
kmin    = (1-dev)*ks;               % lower bound on the grid
kmax    = (1+dev)*ks;               % upper bound on the grid

transk	= 2*(kgrid-kmin)/(kmax-kmin)-1;

n			= 8;
Tk			= [ones(nbk,1) transk];
for i=3:n;
   Tk=[Tk 2*transk.*Tk(:,i-1)-Tk(:,i-2)];
end
b=Tk\kp;

kt	=linspace(kmin,kmax,100)';
trkt=2*(kt-kmin)/(kmax-kmin)-1;
Tk			= [ones(100,1) trkt];
for i=3:n;
   Tk=[Tk 2*trkt.*Tk(:,i-1)-Tk(:,i-2)];
end
plot(kgrid(1:50:1000),kp(1:50:1000),'o',kt,Tk*b,'-')
pause;close
k0=0.5*ks;
nrep=50;
k=zeros(nrep+1,1);
k(1)=k0;
%for t=1:nrep;
%   trkt=2*(k(t)-kmin)/(kmax-kmin)-1;
%   Tk			= [1 trkt];
%   for i=3:n;
%      Tk=[Tk 2*trkt.*Tk(:,i-1)-Tk(:,i-2)];
%   end
%   
%   k(t+1)=Tk*b;
%end

k(1)=k0;
for t=1:nrep;
   k(t+1)=interp1(kgrid,kp,k(t),'linear');
end
y=k(1:nrep).^alpha;
i=k(2:nrep+1)-(1-delta)*k(1:nrep);
c=y-i;

subplot(221);plot(1:nrep,k(1:nrep),1:nrep,ks*ones(nrep,1),'--');
set(gca,'fontname','times','fontsize',12);
xlabel('Time','fontname','times','fontsize',12);
title('Capital stock','fontname','times','fontsize',12);
subplot(222);plot(1:nrep,i,1:nrep,is*ones(nrep,1),'--');
set(gca,'fontname','times','fontsize',12);
xlabel('Time','fontname','times','fontsize',12);
title('Investment','fontname','times','fontsize',12);
subplot(223);plot(1:nrep,y,1:nrep,ys*ones(nrep,1),'--');
set(gca,'fontname','times','fontsize',12);
xlabel('Time','fontname','times','fontsize',12);
title('Output','fontname','times','fontsize',12);
subplot(224);plot(1:nrep,c,1:nrep,cs*ones(nrep,1),'--');
set(gca,'fontname','times','fontsize',12);
xlabel('Time','fontname','times','fontsize',12);
title('Consumption','fontname','times','fontsize',12);
pause;
%print -depsc2 irfdet.eps
close