%
% Solve and simulate the borrowing constraint problem (policy iteration)
%
sigma   = 1.50;								% utility parameter
r   		= 0.02;								% interet rate
beta    = 0.95;								% discount factor
gam		= 0.5;								% replacement ratio
a_star 	= 0;									% Borrowing limit
w			= 1;
p			= 0.8;
PI			= [p 1-p;1-p p];
Om			= [gam*w w];

nba     = 1000;                     % number of data points in the grid
crit    = 1;                        % convergence criterion
iter    = 1;                        % iteration
epsi    = 1e-6;                     % convergence parameter

nbw		= 2;
amin    = a_star;               			% lower bound on the grid
amax    = 10;               			% upper bound on the grid
agrid   = linspace(amin,amax,nba)'; % builds the grid
v       = zeros(nba,nbw);             % value function
util    = zeros(nba,nbw);             % value function
ap0     = repmat(agrid,1,nbw);                    % initial guess on k(t+1)
dr      = zeros(nba,nbw);             % decision rule (will contain indices)


for j=1:nbw;
   c			= repmat((1+r)*agrid'+Om(j),nba,1)-repmat(agrid,1,nba);
   neg		= find(c<=0);
   c(neg)	= NaN;
   eval(['util' int2str(j) '= (c.^(1-sigma)-1)/(1-sigma);'])
   eval(['util' int2str(j) '(neg)= -inf;'])
   clear c neg;
end
%
% Main loop
%
while crit>epsi;
   for j=1:nbw;
      eval(['[vtmp,drtmp]	= max(util' int2str(j) '+beta*repmat(v*PI(j,:)'',1,nba));']);
      dr(:,j)=drtmp';
   end
   
   %
   % decision rules
   %
   ap  = agrid(dr);
   %
   % update the value
   %
   Q  = sparse(nba*nbw,nba*nbw);
   for j=1:nbw;
      for i=1:nba;
      eval(['util(i,j)=util' int2str(j) '(dr(i,j),i);'])
      end;
      Q0  = sparse(nba,nba);
      for i=1:nba;
         Q0(i,dr(i,j)) = 1;
      end
      Q((j-1)*nba+1:j*nba,:)	= kron(PI(j,:),Q0);
   end
   Tv  = (speye(nba*nbw)-beta*Q)\util(:);
   crit= max(max(abs(ap-ap0)));
   disp(sprintf('Iteration # %2d \tCriterion: %g',iter,crit))
   v   = reshape(Tv,nba,nbw);
   ap0 = ap;
   iter= iter+1;
end
%save stopi.mat kgrid kp;



subplot(221);h=plot(agrid,ap);
set(h,'color','k','linewidth',1);
xlabel('a_t','fontname','times','fontsize',12)
title('Next period asset holdings','fontname','times','fontsize',12)
set(gca,'fontname','times','fontsize',12)
c=zeros(nba,nbw);
for j=1:nbw;
   c(:,j)	= (1+r)*agrid+Om(j)-ap(:,j);
end
subplot(222);plot((1+r)*repmat(agrid,1,nbw)+repmat(Om,nba,1)-a_star,c,'color','k','linewidth',1);
xlabel('a_t','fontname','times','fontsize',12)
title('Consumption','fontname','times','fontsize',12)
set(gca,'fontname','times','fontsize',12)
pause
%print -depsc2 drstopi.eps
close

plot(agrid,v,'color','g','linewidth',2);
xlabel('a_t','fontname','times','fontsize',14)
set(gca,'fontname','times','fontsize',14)

pause
%print -depsc2 vfstopi.eps
close

tr_a	= 2*(agrid-amin)/(amax-amin)-1;

n			= 8;
Ta			= [ones(nba,1) tr_a];
for i=3:n;
   Ta=[Ta 2*tr_a.*Ta(:,i-1)-Ta(:,i-2)];
end
ba=Ta\ap;

long=500;
[ws,id]=markov(PI,Om,long,1);

a0=1;

as=zeros(long+1,1);
cs=zeros(long,1);
as(1)=a0;
for t=1:long;
   tr_a=2*(as(t)-amin)/(amax-amin)-1;
   Ta			= [1 tr_a];
   for i=3:n;
      Ta=[Ta 2*tr_a.*Ta(:,i-1)-Ta(:,i-2)];
   end
   
   as(t+1)=Ta*ba(:,id(t));
cs(t)=(1+r)*as(t)+ws(t)-as(t+1);
   
end
subplot(221);plot(1:long,as(1:long));
set(gca,'fontname','times','fontsize',12);
xlabel('Time','fontname','times','fontsize',12);
title('Asset holdings','fontname','times','fontsize',12);
subplot(222);plot(1:long,cs);
set(gca,'fontname','times','fontsize',12);
xlabel('Time','fontname','times','fontsize',12);
title('Consumption','fontname','times','fontsize',12);
pause;
%print -depsc2 simogm.eps
close