clear all
%
% Save the OGM by parametric value iteration
%
sigma   = 1.50;                     % utility parameter
delta   = 0.10;                     % depreciation rate
beta    = 0.95;                     % discount factor
alpha   = 0.30;                     % capital elasticity of output

nbk     = 20;                     	 % number of data points in the grid
p			= 10;									 % order of polynomials
crit    = 1;                        % convergence criterion
iter    = 1;                        % iteration
epsi    = 1e-6;                     % convergence parameter

ks      = ((1-beta*(1-delta))/(alpha*beta))^(1/(alpha-1));
dev     = 0.9;                      % maximal deviation from steady state
kmin    = (1-dev)*ks;               % lower bound on the grid
kmax    = (1+dev)*ks;               % upper bound on the grid
rk			= rcheb(nbk);
kgrid		= kmin+(rk+1)*(kmax-kmin)/2;
v			= (((kgrid.^alpha).^(1-sigma)-1)/((1-sigma)*(1-beta)));;
X			= chebychev(rk,n);
theta0	= X\v
Tv			= zeros(nbk,1);
kp			= zeros(nbk,1);
%
% Main loop
%
options=foptions;
options(14)=1e9;
lb=0;

while crit>epsi;
   k0=kgrid(1);
   for i=1:nbk
      param=[alpha beta delta sigma kmin kmax n kgrid(i)];
      kp(i)=fminu('tv',k0,options,[],param,theta0);
      k0=kp(i);
      Tv(i)=-tv(kp(i),param,theta0);
   end;
   theta=X\Tv;
   crit= max(abs(Tv-v));
   disp(sprintf('Iteration # %2d \tCriterion: %g',iter,crit))
   v=Tv;
   theta0  = theta;
   iter= iter+1;
end
c=kgrid.^alpha+(1-delta)*kgrid-kp;
subplot(221);h=plot(kgrid,kp);
set(h(1),'color','g','linewidth',2);
axis([0 5 0 5])
xlim=get(gca,'xlim');
ylim=get(gca,'ylim');
h=line([ks ks],[ylim(1) ks]);
set(h,'color','k','linestyle','--');
h=line([xlim(1) ks],[ks ks]);
set(h,'color','k','linestyle','--');
h=line(xlim,ylim);
set(h,'color','k','linestyle','-');
xlabel('k_t','fontname','times','fontsize',12)
title('Next period capital stock','fontname','times','fontsize',12)
set(gca,'fontname','times','fontsize',12)
subplot(222);plot(kgrid,c,'color','g','linewidth',2);
xlabel('k_t','fontname','times','fontsize',12)
title('Consumption','fontname','times','fontsize',12)
set(gca,'fontname','times','fontsize',12)
pause
print -depsc2 drparam.eps
close

plot(kgrid,v,'color','g','linewidth',2);
xlabel('k_t','fontname','times','fontsize',14)
set(gca,'fontname','times','fontsize',14)

pause
print -depsc2 valueparam.eps
close