clear all
%
% Solve the OGM by policy iteration + iterations
%
sigma   = 1.50;                     % utility parameter
delta   = 0.10;                     % depreciation rate
beta    = 0.95;                     % discount factor
alpha   = 0.30;                     % capital elasticity of output

nbk     = 1000;                     % number of data points in the grid
crit    = 1;                        % convergence criterion
iter    = 1;                        % iteration
epsi    = 1e-6;                     % convergence parameter

ks      = ((1-beta*(1-delta))/(alpha*beta))^(1/(alpha-1));

dev     = 0.9;                      % maximal deviation from steady state
kmin    = (1-dev)*ks;               % lower bound on the grid
kmax    = (1+dev)*ks;               % upper bound on the grid
devk    = (kmax-kmin)/(nbk-1);      % implied increment
kgrid   = linspace(kmin,kmax,nbk)'; % builds the grid
v       = zeros(nbk,1);             % value function
kp0     = kgrid;                    % initial guess on k(t+1)
dr      = zeros(nbk,1);             % decision rule (will contain indices)
%
% Main loop
%
while crit>epsi;
  for i=1:nbk
    %
    % compute indexes for which consumption is positive
    %
    imin    = max(ceil(((1-delta)*kgrid(i)-kmin)/devk)+1,1);
    imax    = min(floor((kgrid(i)^alpha+(1-delta)*kgrid(i)-kmin)/devk)+1,nbk);
    %
    % consumption and utility
    %
    c       = kgrid(i)^alpha+(1-delta)*kgrid(i)-kgrid(imin:imax);
    util    = (c.^(1-sigma)-1)/(1-sigma);
    %
    % find new policy rule
    %
    [v1,idr]= max(util+beta*v(imin:imax));
    dr(i)   = imin-1+idr;
  end;
    %
    % decision rules
    %
    kp  = kgrid(dr);
    c   = kgrid.^alpha+(1-delta)*kgrid-kp;
    %
    % update the value
    %
    util= (c.^(1-sigma)-1)/(1-sigma);
    Q   = sparse(nbk,nbk);
    for i=1:nbk;
        Q(i,dr(i)) = 1;
    end
    w0=v;
    for i=1:100;
       w0=util+beta*Q*w0;
    end
    Tv=w0;
    crit= max(abs(kp-kp0));
    v   = Tv;
    kp0 = kp;
    iter= iter+1;
end

save detpi.mat kgrid kp c;

subplot(221);h=plot(kgrid,kp);
set(h(1),'color','g','linewidth',2);
axis([0 5 0 5])
xlim=get(gca,'xlim');
ylim=get(gca,'ylim');
h=line([ks ks],[ylim(1) ks]);
set(h,'color','k','linestyle','--');
h=line([xlim(1) ks],[ks ks]);
set(h,'color','k','linestyle','--');
h=line(xlim,ylim);
set(h,'color','k','linestyle','-');
xlabel('k_t','fontname','times','fontsize',12)
title('Next period capital stock','fontname','times','fontsize',12)
set(gca,'fontname','times','fontsize',12)
subplot(222);plot(kgrid,c,'color','g','linewidth',2);
xlabel('k_t','fontname','times','fontsize',12)
title('Consumption','fontname','times','fontsize',12)
set(gca,'fontname','times','fontsize',12)
pause
print -depsc2 drpi.eps
close

plot(kgrid,v,'color','g','linewidth',2);
xlabel('k_t','fontname','times','fontsize',14)
set(gca,'fontname','times','fontsize',14)

pause
print -depsc2 valuepi.eps
close