clear all
%
% Solve the OGM by value iteration
%
sigma	= 1.5;
delta	= 0.1;
beta	= 0.95;
alpha	= 0.30;
ks		= ((1-beta*(1-delta))/(alpha*beta))^(1/(alpha-1));
csy	= 1-alpha*beta*delta/(1-beta*(1-delta));
dev	= 0.9;
kmin	= (1-dev)*ks;
kmax	= (1+dev)*ks;
nbk	= 1000;
devk	= (kmax-kmin)/(nbk-1);
k		= linspace(kmin,kmax,nbk)';
v0		= zeros(nbk,1);
v0		= ((csy*k.^alpha).^(1-sigma)-1)./((1-sigma)*(1-beta));
v		= zeros(nbk,1);
ik1	= zeros(nbk,1);
iter	= 1;
crit	= 1;
tol	= 1e-6;
while crit>tol;
   for i=1:nbk
      imin	= max(ceil(((1-delta)*k(i)-kmin)/devk)+1,1);
      imax	= min(floor((k(i)^alpha+(1-delta)*k(i)-kmin)/devk)+1,nbk);
      c			= k(i)^alpha+(1-delta)*k(i)-k(imin:imax);
      u			= (c.^(1-sigma)-1)/(1-sigma);
      [v(i),itmp]	= max(u+beta*v0(imin:imax));
      ik1(i)	= imin-1+itmp;
   end;
   crit	= max(abs(v-v0));
   v0		= v;
   iter	= iter+1;
end

k1	= k(ik1);
c	= k.^alpha+(1-delta)*k-k1;
u			= (c.^(1-sigma)-1)/(1-sigma);
v=u/(1-beta);

subplot(221);h=plot(k,k1);
set(h(1),'color','g','linewidth',2);
axis([0 5 0 5])
xlim=get(gca,'xlim');
ylim=get(gca,'ylim');
h=line([ks ks],[ylim(1) ks]);
set(h,'color','k','linestyle','--');
h=line([xlim(1) ks],[ks ks]);
set(h,'color','k','linestyle','--');
h=line(xlim,ylim);
set(h,'color','k','linestyle','-');
xlabel('k_t','fontname','times','fontsize',12)
title('Next period capital stock','fontname','times','fontsize',12)
set(gca,'fontname','times','fontsize',12)
subplot(222);plot(k,c,'color','g','linewidth',2);
xlabel('k_t','fontname','times','fontsize',12)
title('Consumption','fontname','times','fontsize',12)
set(gca,'fontname','times','fontsize',12)
pause
print -depsc2 drvi.eps
close

plot(k,v,'color','g','linewidth',2);
xlabel('k_t','fontname','times','fontsize',14)
set(gca,'fontname','times','fontsize',14)

pause
print -depsc2 valuevi.eps
close