%
% solves the OGM by value iteration + interpolation
%
sigma	= 1.5;
delta	= 0.1;
beta	= 0.95;
alpha	= 0.30;
ks		= ((1-beta*(1-delta))/(alpha*beta))^(1/(alpha-1));
csy	= 1-alpha*beta*delta/(1-beta*(1-delta));
dev	= 0.9;
kmin	= (1-dev)*ks;
kmax	= (1+dev)*ks;
nbk=20;
nbc	= 1000;
devk	= (kmax-kmin)/(nbk-1);
k		= linspace(kmin,kmax,nbk)';
v		= zeros(nbk,1);
v		= ((csy*k.^alpha).^(1-sigma)-1)./(1-sigma);
iter	= 1;
crit	= 1;
tol	= 1e-6;
dev     = 0.9;                      % maximal deviation from steady state
kmin    = (1-dev)*ks;               % lower bound on the grid
kmax    = (1+dev)*ks;               % upper bound on the grid
devk    = (kmax-kmin)/(nbk-1);      % implied increment
kgrid   = linspace(kmin,kmax,nbk)'; % builds the grid
cmin    = 0.01;                     % lower bound on the grid
cmax    = kmax^alpha;               % upper bound on the grid
c       = linspace(cmin,cmax,nbc)'; % builds the grid
Tv      = zeros(nbk,1);             % value function
dr      = zeros(nbk,1);             % decision rule (will contain indices)
util    = (c.^(1-sigma)-1)/(1-sigma);
tic
while crit>tol;
   for i=1:nbk;
     kp             = k(i).^alpha+(1-delta)*k(i)-c;
     vi             = interp1(k,v,kp,'spline');
     [Tv(i),dr(i)]  = max(util+beta*vi);
   end
  crit  = max(abs(Tv-v));   % Compute convergence criterion
  v     = Tv;               % Update the value function
   disp(sprintf('Iteration # %2d \tCriterion: %g',iter,crit))
   iter	= iter+1;
end
toc
c	= c(dr);
kp = k.^alpha+(1-delta)*k-c;
u	= (c.^(1-sigma)-1)/(1-sigma);
%v	= u/(1-beta);

subplot(221);h=plot(k,kp);
set(h(1),'color','g','linewidth',2);
axis([0 5 0 5])
xlim=get(gca,'xlim');
ylim=get(gca,'ylim');
h=line([ks ks],[ylim(1) ks]);
set(h,'color','k','linestyle','--');
h=line([xlim(1) ks],[ks ks]);
set(h,'color','k','linestyle','--');
h=line(xlim,ylim);
set(h,'color','k','linestyle','-');
xlabel('k_t','fontname','times','fontsize',12)
title('Next period capital stock','fontname','times','fontsize',12)
set(gca,'fontname','times','fontsize',12)
subplot(222);plot(k,c,'color','g','linewidth',2);
xlabel('k_t','fontname','times','fontsize',12)
title('Consumption','fontname','times','fontsize',12)
set(gca,'fontname','times','fontsize',12)
pause
print -depsc2 drvii.eps
close

plot(k,v,'color','g','linewidth',2);
xlabel('k_t','fontname','times','fontsize',14)
set(gca,'fontname','times','fontsize',14)

pause
print -depsc2 valuevii.eps
close