%
% Solve the stochastic OGM by policy iteration
%
% more efficient
%
sigma   = 1.50;                     % utility parameter
delta   = 0.10;                     % depreciation rate
beta    = 0.95;                     % discount factor
alpha   = 0.30;                     % capital elasticity of output
p		= 0.9;
PI		= [p 1-p;1-p p];
se		= 0.2;
ab		= 0;
am		= exp(ab-se);
as		= exp(ab+se);
A=[am as];
nba=2;


nbk     = 1000;                     % number of data points in the grid
crit    = 1;                        % convergence criterion
iter    = 1;                        % iteration
epsi    = 1e-6;                     % convergence parameter

ks      = ((1-beta*(1-delta))/(alpha*beta))^(1/(alpha-1));

kmin    = 0.2;               			% lower bound on the grid
kmax    = 6;               			% upper bound on the grid
devk    = (kmax-kmin)/(nbk-1);      % implied increment
kgrid   = linspace(kmin,kmax,nbk)'; % builds the grid
v       = zeros(nbk,nba);             % value function
util    = zeros(nbk,nba);             % value function
Ev      = zeros(nbk,nba);             % value function
kp0     = repmat(kgrid,1,nba);                    % initial guess on k(t+1)
dr      = zeros(nbk,nba);             % decision rule (will contain indices)

for j=1:nba;
   c			= repmat(A(j)*kgrid'.^alpha+(1-delta)*kgrid',nbk,1)-repmat(kgrid,1,nbk);
   neg		= find(c<=0);     
   c(neg)	= NaN;
   eval(['util' int2str(j) '= (c.^(1-sigma)-1)/(1-sigma);'])
   eval(['util' int2str(j) '(neg)= -inf;'])
   clear c neg;
end
%
% Main loop
%
while crit>epsi;
   for j=1:nba;
      eval(['[vtmp,drtmp]	= max(util' int2str(j) '+beta*repmat(v*PI(j,:)'',1,nbk));']);
      dr(:,j)=drtmp';
   end
   
   %
   % decision rules
   %
   kp  = kgrid(dr);
   %
   % update the value
   %
   Q  = sparse(nbk*nba,nbk*nba);
   for j=1:nba;
      for i=1:nbk;
      eval(['util(i,j)=util' int2str(j) '(dr(i,j),i);'])
      end;
      Q0  = sparse(nbk,nbk);
      for i=1:nbk;
         Q0(i,dr(i,j)) = 1;
      end
      Q((j-1)*nbk+1:j*nbk,:)	= kron(PI(j,:),Q0);
   end
   Tv  = (speye(nbk*nba)-beta*Q)\util(:);
   crit= max(max(abs(kp-kp0)));
   v   = reshape(Tv,nbk,nba);
   kp0 = kp;
   iter= iter+1;
end
save stopi.mat kgrid kp;


subplot(221);h=plot(kgrid,kp);
set(h,'color','k','linewidth',1);
axis([0 kmax 0 kmax])
xlim=get(gca,'xlim');
ylim=get(gca,'ylim');
h=line(xlim,ylim);
set(h,'color','k','linestyle','--');
xlabel('k_t','fontname','times','fontsize',12)
title('Next period capital stock','fontname','times','fontsize',12)
set(gca,'fontname','times','fontsize',12)
c=zeros(nbk,nba);
for j=1:nba;
   c(:,j)	= A(j)*kgrid.^alpha+(1-delta)*kgrid-kp(:,j);
end
subplot(222);plot(kgrid,c,'color','k','linewidth',1);
xlabel('k_t','fontname','times','fontsize',12)
title('Consumption','fontname','times','fontsize',12)
set(gca,'fontname','times','fontsize',12)
pause
%print -depsc2 drstopi.eps
close

plot(kgrid,v,'color','g','linewidth',2);
xlabel('k_t','fontname','times','fontsize',14)
set(gca,'fontname','times','fontsize',14)

pause
%print -depsc2 vfstopi.eps
close