clear all
%
% Solve the stochastic OGM by value iteration
%
% uses interpolation
%
sigma	= 1.5;
delta	= 0.1;
beta	= 0.95;
alpha	= 0.30;
p		= 0.9;
PI		= [p 1-p;1-p p];
se		= 0.2;
ab		= 0;
am		= exp(ab-se);
as		= exp(ab+se);
A=[am as];
nba=2;

ks		= ((1-beta*(1-delta))/(alpha*beta))^(1/(alpha-1));
csy	= 1-alpha*beta*delta/(1-beta*(1-delta));
dev	= 0.9;
kmin	= 0.2;(1-dev)*ks;
kmax	= 6;(1+dev)*ks;
nbk	= 20;
nbc	= 1000;
devk	= (kmax-kmin)/(nbk-1);
k		= linspace(kmin,kmax,nbk)';
c		= linspace(0.01,as*kmax^alpha,nbc)';
util	= (c.^(1-sigma)-1)/(1-sigma);
v		= repmat(k.^(alpha*(1-sigma))/(1-sigma),1,nba);%zeros(nbk,nba);
Tv		= zeros(nbk,nba);
kp		= zeros(nbc,nba);
iter	= 1;
crit	= 1;
tol	= 1e-6;

while crit>tol;
   for i=1:nbk;
      for j=1:nba;
         kp		= A(j)*k(i).^alpha+(1-delta)*k(i)-c;
         vi		= interp1(k,v,kp,'spline');
         [vtmp,drtmp]	= max(util+beta*(vi*PI(j,:)'));
         Tv(i,j)=vtmp';
         dr(i,j)=drtmp';
      end
   end
   crit	= max(max(abs(Tv-v)));
   v		= Tv;
   iter	= iter+1;
end

kp=zeros(nbk,nba);
ct=zeros(nbk,nba);
for j=1:nba;
	ct(:,j)=c(dr(:,j))
   kp(:,j)	= A(j)*k.^alpha+(1-delta)*k-ct(:,j);
end

subplot(221);h=plot(k,kp);
set(h,'color','k','linewidth',1);
axis([0 kmax 0 kmax])
xlim=get(gca,'xlim');
ylim=get(gca,'ylim');
h=line(xlim,ylim);
set(h,'color','k','linestyle','--');
xlabel('k_t','fontname','times','fontsize',12)
title('Next period capital stock','fontname','times','fontsize',12)
set(gca,'fontname','times','fontsize',12)
subplot(222);plot(k,ct,'color','k','linewidth',1);
xlabel('k_t','fontname','times','fontsize',12)
title('Consumption','fontname','times','fontsize',12)
set(gca,'fontname','times','fontsize',12)
pause
print -depsc2 drstovi.eps
close

plot(k,v,'color','g','linewidth',2);
xlabel('k_t','fontname','times','fontsize',14)
set(gca,'fontname','times','fontsize',14)

pause
print -depsc2 vfstovi.eps
close